<?php

use Millesimes\PaiementComplement;
use Symfony\Component\HttpFoundation\Session\SessionInterface;

/**
 * Class Tpe
 */
class Tpe extends Model {

    protected $table = 'tpe_reference';

    /**
     * @param array $formvars
     *
     * @return array
     */
    function get($formvars = array()) {

        // $sql = "SELECT *, UNCOMPRESS(tpe_reference_session) as tpe_reference_session, UNCOMPRESS(tpe_reference_retour_info) as tpe_reference_retour_info";
        $sql = "SELECT *, tpe_reference_session, UNCOMPRESS(tpe_reference_retour_info) as tpe_reference_retour_info";
        $sql.=" FROM tpe_reference";
        $sql.=" WHERE 1";
        if (!empty($formvars['tpe_reference_id'])) {
            $sql .= " AND tpe_reference_id = '" . $formvars['tpe_reference_id'] . "'";
        }

        if (!empty($formvars['client_id'])) {
            $sql .= " AND client_id = '" . $formvars['client_id'] . "'";
        }

        if (empty($champs)) {
            return $this->sql->getAll($sql);
        } else {
            $result = $this->sql->getOne($sql);
            return $result[$champs];
        }
    }


    /**
     * @param SessionInterface $session
     * @param Panier $panier
     *
     * @return mixed
     * @throws DediException
     */
    public function generateReferenceFromPanier(SessionInterface $session, Panier $panier)
    {

        $data = [
            'paiement_id' => $panier->getPayment()[0]->paiement_id,
            'tpe_reference_session' => serialize($panier),
            'tpe_reference_type' => 'commande',
        ];

        return $this->generateReference($session, $data);
    }

    /**
     * @param SessionInterface $session
     * @param array $data
     *
     * @return mixed
     *
     * @throws DediException
     */
    protected function generateReference(SessionInterface $session, $data = array())
    {

        $data['tpe_reference_date'] = date('Y-m-d H:i:s');
        $data['client_id'] = $session->get('client_id');
        $data['tpe_reference_session_id'] = $session->getId();

        return $this->save($data);
    }

    /**
     * @param $reference
     * @param string $infoRetour
     *
     * @return int
     * @throws DediException
     */
    public function paiementOK($reference, $infoRetour = "")
    {

        // si pas de reference on sort
        if (empty($reference)) {
            throw new DediException('no reference');
        }

        $arrReference = $this->get(array('tpe_reference_id' => $reference));

        // si on trouve pas de reference on sort
        if (count($arrReference) == 0) {
            throw new DediException('no line in table tpe for the reference "' . $reference . '"');
        }

        // si la reference a deja une commande associe on sort
        if (!empty($arrReference[0]['commande_id'])) {
            throw new DediException('the reference "' . $reference . '" already has an order');
        }

        $panier = unserialize($arrReference[0]['tpe_reference_session']);

        // on cree la commande en statut paiement autorise
        $commande = new Commande;
        // try {
        $orderId = $commande->saveFromPanier($panier);
        // } catch (DediException $e) {
        // @TODO : handle error
        // $error->add($e->getMessage());
        // }

        // on associe la commande et on rempli les infos de retour a la reference
        $this->assoc($arrReference[0]['tpe_reference_id'], $orderId);
        $this->addResponse($arrReference[0]['tpe_reference_id'], $infoRetour);

        // Appelle des trackers
        $tracking = new Tracking;
        $tracking->get('paiementActifOk', ['tpe_reference_id' => $arrReference[0]['tpe_reference_id'], 'commande_id' => $orderId]);

        return $orderId;


    }

    /**
     * @param $reference
     * @param array $infoRetour
     *
     * @throws DediException
     */
    public function paiementKO($reference, $infoRetour = array())
    {

        // si pas de reference on sort
        if (empty($reference)) {
            throw new DediException('no reference');
        }

        $arrReference = $this->get(array('tpe_reference_id' => $reference));

        /// si on trouve pas de reference on sort
        if (count($arrReference) == 0) {
            throw new DediException('no line in table tpe for the reference "' . $reference . '"');
        }

        // si la reference a deja une commande associe on sort
        if (!empty($arrReference[0]['commande_id'])) {
            throw new DediException('the reference "' . $reference . '" already has an order');
        }

        // on stock le retour a la reference
        $this->addResponse($reference, $infoRetour);

        // Appelle des trackers
        $tracking = new Tracking;
        $tracking->get('paiementActifKo', array('tpe_reference_id' => $reference));
    }

    // ajout de la reponse a la reference
    public function addResponse($reference, $infoRetour = array())
    {

        $retour = "";
        foreach ($infoRetour as $key => $value) {
            if (!empty($value)) {
                $retour .= $key . " = " . $value . "\n";
            }
        }

        $sql = "UPDATE tpe_reference SET";
        $sql.=" tpe_reference_retour_date = NOW()";
        $sql.=", tpe_reference_retour_info = COMPRESS('" . addslashes($retour) . "')";
        $sql.=" WHERE tpe_reference_id='" . $reference . "'";
        $this->sql->query($sql);
    }

    // ajout de la reponse a la reference
    function assoc($reference, $commande) {

        $sql = "UPDATE tpe_reference SET";
        $sql.=" commande_id='" . $commande . "'";
        $sql.=" WHERE tpe_reference_id='" . $reference . "'";
        $this->sql->query($sql);
    }

    /**
     * @param $reference
     * @param $complement
     *
     * @return mixed
     *
     * @throws DediException
     */
    public function assocComplement($reference, $complement)
    {
        return $this->save([
            'tpe_reference_id' => $reference,
            'paiement_complement_id' => $complement,
        ]);
    }

    // fonction utile pour ajouter une commande depuis le back office
    function response() {

        if (md5($_GET['tpe_reference_id'] . '_em_dedi') != $_GET['key']) {
            throw new InvalidArgumentException(sprintf("Incorrect key given : '%s'", $_GET['key']));
        }

        $this->paiementOK($_GET['tpe_reference_id'], '');
    }

    /**
     * @return Client
     *
     * @throws DediException
     */
    public function getClient()
    {
        if (empty($this->tpe_reference_id)) {
            throw new DediException('no tpe_reference load');
        }

        return new Client($this->client_id);
    }
}